package com.loit.key.generate.server.controller;

import com.loit.component.keygen.leaf.LeafSegmentKeyGeneratorFactory;
import com.loit.key.generate.server.exception.LeafServerException;
import com.loit.key.generate.server.exception.NoKeyException;
import com.sankuai.inf.leaf.IDGen;
import com.sankuai.inf.leaf.common.Result;
import com.sankuai.inf.leaf.common.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class LeafController {

    private Logger logger = LoggerFactory.getLogger(LeafController.class);

    @Autowired
    private LeafSegmentKeyGeneratorFactory leafSegmentKeyGeneratorFactory;


    @Autowired
    private IDGen idGen;

    @RequestMapping(value = "/api/segment/get/{key}")
    public String getSegmentId(@PathVariable("key") String key) {
        Comparable<?> t_order_id = leafSegmentKeyGeneratorFactory.getKeyGenerator(key).generateKey();
        Result result = new Result();
        result.setId((Long) t_order_id);
        return get(key, result);
    }

    @RequestMapping(value = "/api/snowflake/get/{key}")
    public String getSnowflakeId(@PathVariable("key") String key) {
        Result result = idGen.get(key);
        return get(key, result);
    }

    private String get(@PathVariable("key") String key, Result id) {
        Result result;
        if (key == null || key.isEmpty()) {
            throw new NoKeyException("Key is none");
        }
        result = id;
        if (Status.EXCEPTION.equals(result.getStatus())) {
            throw new LeafServerException(result.toString());
        }
        return String.valueOf(result.getId());
    }
}
